"use strict";
//
// Copyright 2023 Signal Messenger, LLC.
// SPDX-License-Identifier: AGPL-3.0-only
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.Net = exports.Environment = void 0;
const Native = require("../Native");
const Address_1 = require("./Address");
const DEFAULT_CHAT_REQUEST_TIMEOUT_MILLIS = 5000;
// This must match the libsignal-bridge Rust enum of the same name.
var Environment;
(function (Environment) {
    Environment[Environment["Staging"] = 0] = "Staging";
    Environment[Environment["Production"] = 1] = "Production";
})(Environment = exports.Environment || (exports.Environment = {}));
class Net {
    constructor(env) {
        this._asyncContext = { _nativeHandle: Native.TokioAsyncContext_new() };
        this._connectionManager = {
            _nativeHandle: Native.ConnectionManager_new(env),
        };
        this._chatService = {
            _nativeHandle: Native.ChatService_new(this._connectionManager, '', ''),
        };
        this.svr3 = new Svr3ClientImpl(this._asyncContext, this._connectionManager);
    }
    async disconnectChatService() {
        await Native.ChatService_disconnect(this._asyncContext, this._chatService);
    }
    async unauthenticatedFetchAndDebug(chatRequest) {
        return await Native.ChatService_unauth_send_and_debug(this._asyncContext, this._chatService, Net.buildHttpRequest(chatRequest), chatRequest.timeoutMillis ?? DEFAULT_CHAT_REQUEST_TIMEOUT_MILLIS);
    }
    async unauthenticatedFetch(chatRequest) {
        return await Native.ChatService_unauth_send(this._asyncContext, this._chatService, Net.buildHttpRequest(chatRequest), chatRequest.timeoutMillis ?? DEFAULT_CHAT_REQUEST_TIMEOUT_MILLIS);
    }
    static buildHttpRequest(chatRequest) {
        const { verb, path, body, headers } = chatRequest;
        const bodyBuffer = body !== undefined ? Buffer.from(body) : null;
        const httpRequest = {
            _nativeHandle: Native.HttpRequest_new(verb, path, bodyBuffer),
        };
        headers.forEach((header) => {
            const [name, value] = header;
            Native.HttpRequest_add_header(httpRequest, name, value);
        });
        return httpRequest;
    }
    async cdsiLookup({ username, password }, { e164s, acisAndAccessKeys, timeout, returnAcisWithoutUaks, }) {
        const request = { _nativeHandle: Native.LookupRequest_new() };
        e164s.forEach((e164) => {
            Native.LookupRequest_addE164(request, e164);
        });
        acisAndAccessKeys.forEach(({ aci: aciStr, accessKey: accessKeyStr }) => {
            Native.LookupRequest_addAciAndAccessKey(request, Address_1.Aci.parseFromServiceIdString(aciStr).getServiceIdFixedWidthBinary(), Buffer.from(accessKeyStr, 'base64'));
        });
        Native.LookupRequest_setReturnAcisWithoutUaks(request, returnAcisWithoutUaks);
        const lookup = await Native.CdsiLookup_new(this._asyncContext, this._connectionManager, username, password, request, timeout);
        return await Native.CdsiLookup_complete(this._asyncContext, {
            _nativeHandle: lookup,
        });
    }
}
exports.Net = Net;
class Svr3ClientImpl {
    constructor(_asyncContext, _connectionManager) {
        this._asyncContext = _asyncContext;
        this._connectionManager = _connectionManager;
    }
    async backup(what, password, maxTries, auth, opTimeoutMs) {
        return Native.Svr3Backup(this._asyncContext, this._connectionManager, what, password, maxTries, auth.username, auth.password, opTimeoutMs);
    }
    async restore(password, shareSet, auth, opTimeoutMs) {
        return Native.Svr3Restore(this._asyncContext, this._connectionManager, password, shareSet, auth.username, auth.password, opTimeoutMs);
    }
}
//# sourceMappingURL=net.js.map